#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------

from utils import runShellCommand, ynask, reverse, writefile, killfiles, downloader
import sys
import os
import getopt
import string

class Frame:
    """ This class takes care of the BLACS installation. """
    
    # set default values
    prefix = ''
    ccflags, fcflags, noopt = ['-O3','-O3','']
    mpibindir = ''               # directory containing mpicc and mpif77
    mpiincdir = ''               # directory containing mpi.h
    mpicc, mpif77 = ['', '']     # the mpicc and mpif77 commands
    make         = 'make'        # the "make" command
    plat         = 'LINUX'       
    mangling     = ''            # sets the mangling type for Fortran to C calls
    blaslib      = ''            # the blas library
    blacslib     = ''            # the blacs library
    blacsClib    = ''            # the blacs C library
    blacsF77lib  = ''            # the blacs F77 library
    lapacklib    = ''            # lapack library
    scalapacklib = ''            # scalapack library
    proxy        = 0
    downblas     = 0             # whether or not to download reference blas
    downblacs    = 1             # whether or not to download reference blacs
    downlapack   = 0             # whether or not to download reference lapack
    ldflags_c    = ''            # loader flags when main program is in C
    ldflags_f77  = ''            # loader flags when main program is in Fortran
    testing      = 1             # whether on not to compile and run LAPACK and ScaLAPACK test programs
    downcmd      = ''            # the command used to download stuff
    blasurl      = 'http://netlib.org/blas/blas.tgz'
    lapackurl    = 'http://netlib.org/lapack/lapack-lite-3.1.1.tgz'
    blacsurl     = 'http://netlib.org/blacs/mpiblacs.tgz'
    scalapackurl = 'http://netlib.org/scalapack/scalapack-1.8.0.tgz'
    ranlib       = ''            # the ranlib command
    clean        = 0

    def __init__(self, argv):
        
        print '='*40
        print 'Setting up the framework\n'


        # parse input arguments
        self.parse_args(argv)
        
        if(Frame.clean == 1):
            self.cleanup()
            sys.exit()

        if(Frame.prefix != ''):
	        if(not os.path.isdir(os.path.join(os.getcwd(),Frame.prefix))):
	           print"Creating directory",Frame.prefix
	           os.mkdir(os.path.join(os.getcwd(),Frame.prefix))
	        os.chdir(Frame.prefix)
	
        if(Frame.mpicc=='' or Frame.mpif77==''):
            #  if no MPI compilers are provided, look for them in mpibindir or in PATH
            if(not self.look_for_mpibinaries()):
                print """
MPI C and Fortran77 compilers are needed  to complete
the installation. You can specify the using the --mpibindir
or the --mpicc and --mpif77 flags."""
                sys.exit()
                
        if(Frame.mpiincdir==''):
            # the path to the mpi.h file is needed
            if(not self.look_for_mpih()):
                print 'Please provide the location for mpi.h file using the --mpiincdir flag'
                sys.exit()

        # if testing has to be done, BLAS, BLACS andLAPACK are required
        if(Frame.testing == 1):
            if(Frame.blaslib == '' and Frame.downblas == 0):
                print """
Please provide a working BLAS library. If a BLAS library
is not present on the system, the reference BLAS library it can be
automatically downloaded and installed by adding the --downblas flag.
Be aware that a reference BLAS library will be installed with the --downblas
flag so don't expect performance.
The BLAS library is not needed in the case where testing is disabled
by means of the --notesting flag. 
                """
                sys.exit()
            if((Frame.blacslib == '' or Frame.blacsClib == '' or Frame.blacsF77lib == '') and Frame.downblacs == 0):
                print """
Please provide a working BLACS library. If a BLACS library
is not present on the system, it can be automatically downloaded and
installed by adding the --downblacs flag. The BLAS library is not
needed in the case where testing is disabled by means of the
--notesting flag.
                """
                sys.exit()
            if(Frame.lapacklib == '' and Frame.downlapack == 0):
                print """
Please provide a working LAPACK library. If a LAPACK library
is not present on the system, it can be automatically downloaded and
installed by adding the --downlapack flag. The BLAS library is not
needed in the case where testing is disabled by means of the
--notesting flag.
                """
                sys.exit()
        
          
        # CUSTOM CHECKS
        self.check_mpicc()
        self.check_mpif77()
        self.set_mangling()
        self.set_download()
        self.set_ranlib()
        self.detect_compilers()
        self.check_linking()
        
        return


    def usage(self):
          print '='*40
          print """
          ScaLAPACK configuration script
          The script will try to figure out some of the features of your system
          like the mpi compiler and the location of few other tools required for
          the installation of the ScaLAPACK package and the other software
          packages that it requires.
          It is strongly recommended that you help the script by providing info 
          through the flags listed below

          
          -h or --help         : prints this message
 
          --prefix             : path where to create the libraries, build and log of the installer

          --mpibindir=[DIR]    : the path to where the mpi
                                 binaries (mpicc and mpif77)
                                 are contained

          --mpicc=[CMD]        : the mpicc command.
                                 (default: will take the first available mpicc in the path)

          --mpif77=[CMD]       : the mpif77 command.
                                 (default: will take the first available mpif77 in the path)

          --mpiincdir=[DIR]    : the path to the directory containing mpi.h
                                 (default: will take the corresponding mpi.h to the mpicc/mpif77 found)

          --ccflags=[FLAGS]    : the flags for the C compiler
                                 (default -O3)

          --fcflags=[FLAGS]    : the flags for the F77 compiler
                                 (default -O3)

          --noopt=[FLAGS]      : compilation flags to be used
                                 on machine dependent subroutines
                                 in LAPACK and ScaLAPACK.
                                 See LAPACK and ScaLAPACK documentation.

          --ldflags_c=[flags]  : loader flags when main program is in C. Some compilers (e.g. PGI) require 
                                 different options when linking C main programs to Fortran subroutines
                                 and vice-versa

          --ldflags_f77=[flags]: loader flags when main program is in Fortran. Some compilers (e.g. PGI) require 
                                 different options when linking Fortran main programs to C subroutines
                                 and vice-versa

          --makecmd=[CMD]      : the make command
                                 (make by default)

          --blacslib=[LIB]     : the BLACS library
                                 (path should be absolute if --prefix is used)

          --blacsclib=[LIB]    : the BLACS C interface library
                                 (path should be absolute if --prefix is used)

          --blacsf77lib=[LIB]  : the BLACS F77 interface library
                                 (path should be absolute if --prefix is used)

          --blaslib=[LIB]      : a BLAS library
                                 (path should be absolute if --prefix is used)

          --lapacklib=[LIB]    : a LAPACK library
                                 (path should be absolute if --prefix is used)

          --downblas           : if you do not want to provide a BLAS
                                 we can download and install it for you

          --downblacs          : if you do not want to provide a BLACS
                                 we can download and install it for you

          --downlapack         : if you do not want to provide a LAPACK
                                 we can download and install it for you

          --notesting          : disables the ScaLAPACK testing. The
                                 BLAS, BLACS and LAPACK libraries are not
                                 required in this case.

          --clean              : cleans up the installer directory.

Note: If you want use a proxy for downloading, the http_proxy environment variable has to be set.
          """
          print '='*40
          sys.exit()
      


    def parse_args(self, argv):
        """ Parse input argument to get compilers etc. from command line. """

        try:
          opts, args = getopt.getopt(sys.argv[1:], "h", ["help","prefix=",
          "ccflags=", "fcflags=", "noopt=", "makecmd=", "mpibindir=", "mpiincdir=", "blacslib=",
          "blacsclib=", "blacsf77lib=", "blaslib=", "lapacklib=", "ldflags_c=", "ldflags_f77=","mpicc=","mpif77=",
          "downblas", "downblacs", "downlapack", "notesting","clean"])
        except getopt.error, msg:
          print msg
          print "for help use --help"
          sys.exit(2)
        # process options
        for o, a in opts:
          if o in ("-h", "--help"):
            self.usage()
            sys.exit(0)
          else:
            if(o == '--clean'):
                Frame.clean = 1
                return;
            elif(o == '--prefix'):
                Frame.prefix = a
                print 'Install directory is...', a
            elif(o == '--ccflags'):
                Frame.ccflags = a
                print 'C flags are ', a
            elif(o=='--fcflags'):
                Frame.fcflags = a
                print 'Fortran flags are ', a
            elif(o=='--noopt'):
                Frame.noopt = a
                print 'NOOPT flags are ', a
            elif(o=='--makecmd'):
                Frame.make = a
            elif(o=='--mpibindir'):
                Frame.mpibindir = a
                print 'MPI bin dir is ', a
            elif(o=='--mpiincdir'):
                Frame.mpiincdir = a
                print 'MPI include dir is ', a
            elif(o=='--mpicc'):
                Frame.mpicc = a
                print 'mpicc is ', a
            elif(o=='--mpif77'):
                Frame.mpif77 = a
                print 'mpif77 is ', a
            elif(o == '--blacslib'):
                Frame.blacslib = a
                Frame.downblacs = 0
            elif(o == '--blacsclib'):
                Frame.blacsClib = a
                Frame.downblacs = 0
            elif(o == '--blacsf77lib'):
                Frame.blacsF77lib = a
                Frame.downblacs = 0
            elif (o == '--blaslib'):
                Frame.blaslib = a
            elif (o == '--lapacklib'):
                Frame.lapacklib = a
            elif (o == '--downblas'):
                Frame.downblas = 1
            elif (o == '--downblacs'):
                Frame.downblacs = 1
            elif (o == '--downlapack'):
                Frame.downlapack = 1
            elif (o == '--notesting'):
                Frame.testing = 0
            elif (o == '--ldflags_c'):
                Frame.ldflags_c = a
            elif (o == '--ldflags_f77'):
                Frame.ldflags_f77 = a

 
    # look for MPI
    def look_for_mpibinaries(self):
        """ looks for MPI compilers in mpibindir or in PATH """
        # This function is only able to find mpicc/mpif77. If the name of the 
        # compilers are different (like mpixlc or mpixlf) the user must explicitly provide them
        # with the related flags        
        if(Frame.mpibindir != ''):
            # look for mpicc and mpif77 in mpibindir  
            if(os.path.isfile(os.path.join(Frame.mpibindir,'mpicc'))):
                Frame.mpicc = os.path.join(Frame.mpibindir,'mpicc')
                print 'mpicc :',Frame.mpicc
            else:
                print 'Could not find mpicc in ', Frame.mpibindir
            if(os.path.isfile(os.path.join(Frame.mpibindir,'mpif77'))):
                Frame.mpif77 = os.path.join(Frame.mpibindir,'mpif77')
                print 'mpif77 :',Frame.mpif77
            else:
                print 'Could not find mpif77 in ',Frame.mpibindir

        # is mpicc and mpif77 haven't been found
        if(Frame.mpicc=='' and Frame.mpif77==''):

            path=str(os.getenv('PATH')).split(os.pathsep)
            print 'Looking for MPI binaries...',
            sys.stdout.flush()
            for i in path:
                mpicc  = os.path.join(i,'mpicc')
                mpif77 = os.path.join(i,'mpif77')
                if (Frame.mpicc == '' and os.path.isfile(mpicc)):
                    Frame.mpicc = mpicc
                if (Frame.mpif77 == '' and os.path.isfile(mpif77)):
                    Frame.mpif77 = mpif77
            if (Frame.mpicc!='' or Frame.mpif77!=''):
                print "mpicc and mpif77 found.\nmpicc : "+Frame.mpicc+"\nmpif77 :"+Frame.mpif77
                return 1;
                    
            elif (Frame.mpicc=='' or Frame.mpif77==''):
                print """
The information about the location of MPI commands is incomplete.
Please, either provide the path to the directory containing mpicc
and mpif77 with the --mpibindir flag or the full path to both 
commands with the --mpicc AND --mpif77 flags. In case none of these
flags are provided, the installer will look for mpicc and mpif77
in the PATH environment variable.
"""
                return 0;

        return 1;




    def look_for_mpih(self):
        """ looks for mpi.h close to mpibindir """
        import re
      
        print 'Looking for mpi.h...',
        for i in [Frame.mpibindir[0:Frame.mpibindir.find('bin')], Frame.mpicc[0:Frame.mpicc.find('bin')], Frame.mpif77[0:Frame.mpif77.find('bin')]]:
            sys.stdout.flush()
            tmp = i+'include'
            if(os.path.isfile(os.path.join(tmp,'mpi.h'))):
                Frame.mpiincdir = tmp
                print 'found in '+i+'include'
                return 1;

        print 'not found'

        return 0;


            
    def check_mpicc(self):
        """ checks if mpicc works """
        # simply generates a C program containing a couple of calls
        # to MPI routines and checks if the compilation and execution
        # are succesful
        print 'Checking if mpicc works...',
        sys.stdout.flush()
        # generate
        writefile('tmpc.c',"""
            #include \"mpi.h\"
            #include <stdio.h>
            int main(int argc, char **argv){
            int iam;
            MPI_Init( &argc, &argv );
            MPI_Comm_rank( MPI_COMM_WORLD, &iam );
            if(iam==0){fprintf(stdout, \"success\" );fflush(stdout);}
            MPI_Finalize();
            return 0;
            }\n""")

        # compile
        ccomm = Frame.mpicc+' -o tmpc '+os.path.join(os.getcwd(),'tmpc.c')
        (output, error, retz) = runShellCommand(ccomm)
      
        if(retz != 0):
            print '\n\nCOMMON: mpicc not working! aborting...'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        # run
        comm = './tmpc'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nCOMMON: mpicc not working! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        # cleanup
        killfiles(['tmpc.c','tmpc'])
        print 'yes'
        return 0;



    def check_mpif77(self):
        """ check if mpif77 works """
        # simply generates a F77 program containing a couple of calls
        # to MPI routines and checks if the compilation and execution
        # are succesful
        print 'Checking if mpif77 works...',
        sys.stdout.flush()
        # generate        
        writefile('tmpf.f',"""
      program ftest
      include 'mpif.h'
      integer Iam, i, ierr
      call mpi_init(ierr)
      call mpi_comm_rank(MPI_COMM_WORLD, Iam, ierr)
      if (Iam .eq. 0) then
            print*,'success'
      end if
      call mpi_finalize(ierr)
      stop
      end\n""")

        # compile
        fcomm = Frame.mpif77+' -o tmpf '+'tmpf.f'
        (output, error, retz) = runShellCommand(fcomm)

        if(retz != 0):
            print '\n\nCOMMON: mpif77 not working! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        # run
        comm = './tmpf'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nCOMMON: mpif77 not working! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        # cleanup        
        killfiles(['tmpf.f','tmpf'])
        print 'yes'

        return 0;
        

    def set_mangling(self):
        """ Sets the INTFACE variable in Bmake.inc """
        # This one generates a program equivalent to that in BLACS/INSTALL
        # that checks the mangling in FORTRAN function symbols
        print 'Setting Fortran mangling...',
        sys.stdout.flush()
        writefile('tmpf.f',"""
      program intface
      external c_intface
      integer i
      call c_intface(i)
      stop
      end\n""")
        writefile('tmpc.c',"""
      #include <stdio.h>
      void c_intface_(int *i){fprintf(stdout, \"-DAdd_\");fflush(stdout);}
      void c_intface(int *i){fprintf(stdout, \"-DNoChange\");fflush(stdout);}
      void c_intface__(int *i){fprintf(stdout, \"-Df77IsF2C\");fflush(stdout);}
      void C_INTFACE(int *i){fprintf(stdout, \"-DUpCase\");fflush(stdout);}\n""")

        ccomm = Frame.mpicc+' '+Frame.ccflags+' -c tmpc.c -o tmpc.o'
        fcomm = Frame.mpif77+' '+Frame.fcflags+' tmpf.f tmpc.o -o xintface'

        (output, error, retz) = runShellCommand(ccomm)
        if(retz != 0):
            print '\n\nCOMMON: in set_mangling: cannot compile'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        (output, error, retz) = runShellCommand(fcomm)
        if(retz != 0):
            print '\n\nCOMMON: in set_mangling: cannot compile'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        comm = os.path.join(os.getcwd(),'xintface')
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nCOMMON: in set_mangling: cannot run xintface'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        Frame.mangling = output
        killfiles(['xintface', 'tmpf.f', 'tmpf.o', 'tmpc.c', 'tmpc.o'])
        
        print Frame.mangling
        return 1;


    def check_linking(self):
        """ Check if C main can be linked to F77 subroutine """

        # This one checks if the linking command works out of the box or
        # if any specific flag is required. For example if the linker if the
        # Intel FORTRAN compiler, then the "-nofor_main" is usually required.
        # This function only checks if linker works but does not automatically
        # detect the required flags
        print 'Checking loader...',
        sys.stdout.flush()
        writefile('tmpf.f',"""
      subroutine fsub()
      write(*,*)'success'
      stop
      end\n""")
        writefile('tmpc.c',"""
      #if defined Add_
      #define fsub fsub_
      #elif defined NoChange
      #define fsub fsub
      #elif defined f77IsF2C
      #define fsub fsub_
      #elif defined UpCase
      #define fsub FSUB
      #endif
      void main(){
      fsub();}\n""")

        ccomm = Frame.mpicc+' '+Frame.ccflags+' '+Frame.mangling+' -c -o tmpc.o tmpc.c'
        fcomm = Frame.mpif77+' '+Frame.fcflags+' -c -o tmpf.o tmpf.f'
        lcomm = Frame.mpif77+' '+Frame.ldflags_c+' -o lnk tmpf.o tmpc.o'
      
        (output, error, retz) = runShellCommand(ccomm)
        if(retz != 0):
            print '\n\nCOMMON: in check_linking: cannot compile'
            print 'command is: ',ccomm
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        (output, error, retz) = runShellCommand(fcomm)
        if(retz != 0):
            print '\n\nCOMMON: in check_linking: cannot compile'
            print 'command is: ',fcomm
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        (output, error, retz) = runShellCommand(lcomm)
        if(retz != 0):
            print """\n\nCOMMON: in check_linking: cannot link
            Cannot link a C main program to a Fortran77 subroutine
            Make sure that the appropriate flags are passed to the linker."""
            print 'command is: ',lcomm
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()


        killfiles(['lnk', 'tmpf.f', 'tmpf.o', 'tmpc.c', 'tmpc.o'])
        
        print 'works'
        return 1;


          
    def set_download(self):
        """ Figures out how to download files """
        print 'Setting download command...',
        # look for proxy setting in the environnement variables
        proxy_http=str(os.getenv('http_proxy')) 
		# if proxy_http found in the environnement, we are going to use wget as download command
        if (proxy_http):
		   Frame.proxy=1
		   urllib=0
        else:    
           try:
            # check if the urllib2 module is included in the
            # python installation. If yes files are downloaded with urllib2            
               import urllib2
               urllib=1
               Frame.downcmd = 'urllib2'
               print Frame.downcmd
               return
           except ImportError:
               urllib=0

        if(not urllib):
            # if urllib2 is not present checks if wget is present
            # in the PATH and if yes it sets the download command
            # to be wget
            path=str(os.getenv('PATH')).split(os.pathsep)
            for i in path:
                if (os.path.isfile(os.path.join(i,'wget'))):
                    Frame.downcmd='wget'
                    print Frame.downcmd
                    if (proxy_http):
					   print 'Using the following proxy to download: ',proxy_http
					   print 'Testing download via proxy...',
					   comm = 'wget http://www.netlib.org/lapack/lapack-3.1.1.changes'
					   (output, error, retz) = runShellCommand(comm)
					   if(retz != 0):
						  print 'not working...aborting'
						  print 'Please change your proxy settings:', proxy_http, 'is not working'
						  sys.exit()
					   print 'working'
					   os.remove('lapack-3.1.1.changes')
                    return
            

    def set_ranlib(self):
        """ Sets the ranlib command """
        # Some systems don't have the ranlib command (e.g. SGIs). 
        # In the case where ranlib is not present in the PATH,
        # echo is used instead of ranlib        
        print "Setting ranlib command...",

        path=str(os.getenv('PATH')).split(os.pathsep)
        for i in path:
            if (os.path.isfile(os.path.join(i,'ranlib'))):
                Frame.ranlib=os.path.join(i,'ranlib')
                print Frame.ranlib
                return

        for i in path:
            if (os.path.isfile(os.path.join(i,'echo'))):
                Frame.ranlib=os.path.join(i,'echo')
                print Frame.ranlib
                return





    def detect_compilers(self):
        """ Tries to detect the compilers type """
        # By users experience it is known which compiler flags are required
        # in some cases. This function tries to detect which compilers are used
        # and sets the flags accordingly

        print 'Detecting Fortran compiler...',
        if (self.fc_is_intel()):
            # The Intel FORTRAN compiler requires -nofor_main flag
            # for the linking and the -mp flag to maintain the 
            # floating-point precision
            Frame.ldflags_c   += ' -nofor_main'
            Frame.ldflags_f77 += ''
            Frame.noopt += ' -mp'
            print 'Intel'
        elif (self.fc_is_gnu()):
            print 'GNU'
        elif (self.fc_is_pgi()):
            Frame.ldflags_c   += ' -Mnomain'
            Frame.ldflags_f77 += ''
        else:
            print 'unknown'


        print 'Detecting C compiler...',
        if (self.cc_is_intel()):
            print 'Intel'
        elif (self.cc_is_gnu()):
            print 'GNU'
        elif (self.cc_is_pgi()):
            print 'PGI'
        else:
            print 'unknown'


        print 'Selected C compiler flags: '+self.ccflags
        print 'Selected Fortran77 compiler flags: '+self.fcflags
        print 'Selected loader flags (C main): '+self.ldflags_c
        print 'Selected loader flags (F77 main): '+self.ldflags_f77
        print 'Selected NOOPT flags: '+self.noopt

        return


    def fc_is_intel(self):
        comm = self.mpif77+' -V'
        (output, error, retz) = runShellCommand(comm)
        isifort = string.find(error,'Intel(R) Fortran Compiler')
        if (isifort != -1):
            return 1

        return 0


    def fc_is_gnu(self):
        comm = self.mpif77+' --help'
        (output, error, retz) = runShellCommand(comm)
        isifort = string.find(error,'gnu.org')
        if (isifort != -1):
            return 1

        return 0

    def fc_is_pgi(self):
        comm = self.mpif77+' -V'
        (output, error, retz) = runShellCommand(comm)
        isifort = string.find(error,'pgif77')
        if (isifort != -1):
            return 1
        isifort = string.find(error,'pgif90')
        if (isifort != -1):
            return 1
        isifort = string.find(error,'pgif95')
        if (isifort != -1):
            return 1
        isifort = string.find(error,'Portland')
        if (isifort != -1):
            return 1
        isifort = string.find(output,'pgif77')
        if (isifort != -1):
            return 1
        isifort = string.find(output,'pgif90')
        if (isifort != -1):
            return 1
        isifort = string.find(output,'pgif95')
        if (isifort != -1):
            return 1
        isifort = string.find(output,'Portland')
        if (isifort != -1):
            return 1

        return 0



    def cc_is_intel(self):
        comm = self.mpicc+' -V'
        (output, error, retz) = runShellCommand(comm)
        isifort = string.find(error,'Intel(R) C Compiler')
        if (isifort != -1):
            return 1

        return 0


    def cc_is_gnu(self):
        comm = self.mpicc+' --help'
        (output, error, retz) = runShellCommand(comm)
        isifort = string.find(error,'gnu.org')
        if (isifort != -1):
            return 1

        return 0
        

    def cc_is_pgi(self):
        comm = self.mpicc+' -V'
        (output, error, retz) = runShellCommand(comm)
        isifort = string.find(error,'pgicc')
        if (isifort != -1):
            return 1
        isifort = string.find(error,'Portland')
        if (isifort != -1):
            return 1
        isifort = string.find(output,'pgicc')
        if (isifort != -1):
            return 1
        isifort = string.find(output,'Portland')
        if (isifort != -1):
            return 1

        return 0


    def resume(self):
        
        cwd = os.getcwd()
        print """
******************************************************
******************************************************

ScaLAPACK installation completed.


Your BLAS library is:\n"""+self.blaslib+"""
\nYour BLACS libraries are:\n"""+self.blacslib+'\n'+self.blacsClib+'\n'+self.blacsF77lib+"""
\nYour LAPACK library is:\n"""+self.lapacklib+"""\n
Your ScaLAPACK library is:\n"""+self.scalapacklib+"""

Log messages are in the\n"""+os.path.join(cwd,'log')+"""
directory.

The Scalapack testing programs are in:\n"""+os.path.join(cwd,'build/scalapack-1.8.0/TESTING')+"""


The\n"""+os.path.join(cwd,'build')+"""
directory contains the source code of the libraries 
that have been installed. It can be removed at this time.


******************************************************
******************************************************
"""



    def cleanup(self):
        """ Cleans up the installer directory """

        print "Cleaning up...",
        sys.stdout.flush()
        
        cwd = os.getcwd()
        builddir = os.path.join(cwd,'build')
        libdir   = os.path.join(cwd,'lib')
        logdir   = os.path.join(cwd,'log')

        comm = 'rm -rf '+builddir+' '+libdir+' '+logdir
        (output, error, retz) = runShellCommand(comm)
        
        print "done."



#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------


import sys
import os
import string
import re
import getopt


def writefile(fname, fill):
    """ writes the file fname with content fill """
    fp = open(fname,'w')
    fp.write(fill)
    fp.close()

def killfiles(lst):
    """ deletes a list of files """
    for i in lst:
        if(os.path.isfile(i)):
            os.remove(i)


def reverse(retto):
    """ reverses a string """
    verso = list(retto)        # string -> list of chars
    verso.reverse()              # inplace reverse the list
    verso = ''.join(verso)
    return verso

def openPipe(command):

    import popen2

    pipe = None
    if hasattr(popen2, 'Popen3'):
        pipe   = popen2.Popen3(command, 1)
        input  = pipe.tochild
        output = pipe.fromchild
        err    = pipe.childerr
    else:
        import os
        (input, output, err) = os.popen3(command)

    return (input, output, err, pipe)


def runShellCommand(command):
    """ runs a shell command """

    import select

    ret        = None
    out        = ''
    err        = ''
    (input, output, error, pipe) = openPipe(command)
    input.close()
    outputClosed = 0
    errorClosed  = 0
    lst = [output, error]
    while 1:
      ready = select.select(lst, [], [])
      if len(ready[0]):
        if error in ready[0]:
          msg = error.readline()
          if msg:
            err += msg
          else:
            errorClosed = 1
            lst.remove(error)
        if output in ready[0]:
          msg = output.readline()
          if msg:
            out += msg
          else:
            outputClosed = 1
            lst.remove(output)
      if outputClosed and errorClosed:
        break
    output.close()
    error.close()
    if pipe:
      ret = pipe.wait()
    return (out, err, ret)



def getURLName(url):
    directory=os.curdir

    name="%s%s%s" % (
        directory,
        os.sep,
        url.split("/")[-1]
	)

    return name



def downloader(uri,cmd):
    """ downloads the content of an URL """

    name = getURLName(uri)
    try:
        if(cmd == 'urllib2'):
            import urllib2 
            url = urllib2.urlopen(uri)
            f = open(name,'w')
            for line in url.readlines():
                f.write(line)
            url.close()
            f.close()
        elif(cmd == 'wget'):
            comm = 'wget '+uri
            (output, error, retz) = runShellCommand(comm)
        else:
            raise
    except:
        print "Cannot download"+name
        print "Make sure the network is reachable."
        print "If you still have troubles, you can manually download bacs.tgz from this URL:"
        print uri
        print "into the current directory:"
        print os.getcwd()
        


def ynask(quest):
    """ asks a binary question """
    print quest
    ans = ''
    while(ans != 'y' and ans != 'n'):
        ans = raw_input("[y/n] :")
        if (ans != 'y' and ans != 'n'):
            print "Please answer \"y\" for yes or \"n\" for no"

    return ans



#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------


from utils import writefile, runShellCommand, killfiles, downloader, getURLName
import sys
import os
from framework import Frame


class Lapack(Frame):
    """ This class takes care of the LAPACK installation. """
    def __init__(self):
        print '\n','='*40
        print '  Lapack installation/verification'
        print '='*40

        if(self.lapacklib != ''):
            self.check_lapack()
        else:
            if (self.downlapack):
                self.down_install_lapack()
            else:
                print "Need LAPACK"
                sys.exit()





    def check_lapack(self):
        """ Checks if the provided LAPACK library is good """
        print 'Checking if provided LAPACK works...',

        # This function simply generates a FORTRAN program
        # that contains few calls to LAPACK routines and then
        # checks if compilation, linking and execution are succesful

        sys.stdout.flush()
        writefile('tmpf.f',"""
      program ftest
      integer info, ipiv(1)
      double precision A(1)
      A(1)=1
      call dgetrf(1,1,A,1,ipiv,info)
      stop
      end\n""")

        fcomm = self.mpif77+' -o tmpf '+'tmpf.f '+self.lapacklib+' '+self.blaslib+' '+self.ldflags_f77
        (output, error, retz) = runShellCommand(fcomm)

        if(retz != 0):
            print '\n\nLAPACK: provided LAPACK cannot be used! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        comm = './tmpf'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nLAPACK: provided LAPACK cannot be used! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        killfiles(['tmpf.f','tmpf'])
        print 'yes'

        return 0;




    def down_install_lapack(self):
        print """ Download and install LAPACK from netlib.org"""

        savecwd = os.getcwd()

        # creating the build and lib dirs if don't exist
        if(not os.path.isdir(os.path.join(os.getcwd(),'build'))):
            os.mkdir(os.path.join(os.getcwd(),'build'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'lib'))):
            os.mkdir(os.path.join(os.getcwd(),'lib'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'log'))):
            os.mkdir(os.path.join(os.getcwd(),'log'))

        # chdir into the build directory            
        os.chdir(os.path.join(os.getcwd(),'build'))

        if(not os.path.isfile(os.path.join(os.getcwd(),getURLName(self.lapackurl)))):
            print """Download LAPACK...""",
            sys.stdout.flush()
            downloader(self.lapackurl, self.downcmd)
            print "done"
	        
	    print """Unzip and untar LAPACK...""",
        comm = 'gunzip -f lapack-lite-3.1.1.tgz'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nLAPACK: cannot unzip lapack.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        comm = 'tar xf lapack-lite-3.1.1.tar'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nLAPACK: cannot untar lapack.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        os.remove('lapack-lite-3.1.1.tar')
        print "done"

        # change to LAPACK dir
        os.chdir(os.path.join(os.getcwd(),'lapack-lite-3.1.1'))

        # compile and generate library
        self.set_etime()
        self.write_makeinc()

        print 'Compiling LAPACK (this will take several minutes)...',
        sys.stdout.flush()
        comm = self.make+' lib'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nLAPACK: error building LAPACK'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        # write the log on a file
        fulllog = os.path.join(savecwd,'log/laplog')
        writefile(fulllog, output+error)
        print 'Installation of LAPACK successful.'
        print '(log is in ',fulllog,')'

        # move librefblas.a to the lib directory        
        os.rename('libreflapack.a',os.path.join(savecwd,'lib/libreflapack.a'))

        # set framework variables to point to the freshly installed BLAS library
        self.lapacklib  = os.path.join(savecwd,'lib/libreflapack.a ')
        Frame.lapacklib = os.path.join(savecwd,'lib/libreflapack.a ')
        os.chdir(savecwd)

        return







    def set_etime(self):
        """ Detects the FORTRAN timer """

        print 'Setting ETIME...',
        writefile('tmpf.f',"""
      PROGRAM DSECND
      REAL               T1
      REAL               TARRAY( 2 )
      REAL               ETIME_
      EXTERNAL           ETIME_
      T1 = ETIME_( TARRAY )
      END\n""")
        fcomm = self.mpif77+' '+self.fcflags+' -o tmpf tmpf.f'
        (output, error, retz) = runShellCommand(fcomm)

        if(retz == 0):
            self.etime = 'EXT_ETIME_'
            print self.etime
            killfiles(['tmpf.f', 'tmpf'])
            return


        writefile('tmpf.f',"""
      PROGRAM DSECND
      REAL               T1
      REAL               TARRAY( 2 )
      REAL               ETIME
      EXTERNAL           ETIME
      T1 = ETIME( TARRAY )
      END\n""")
        
        fcomm = self.mpif77+' '+self.fcflags+' -o tmpf tmpf.f'
        (output, error, retz) = runShellCommand(fcomm)

        if(retz == 0):
            self.etime = 'EXT_ETIME'
            print self.etime
            killfiles(['tmpf.f', 'tmpf'])
            return


        writefile('tmpf.f',"""
      PROGRAM DSECND
      REAL               T
      INTRINSIC          ETIME
      CALL CPU_TIME( T )
      END\n""")
        
        fcomm = self.mpif77+' '+self.fcflags+' -o tmpf tmpf.f'
        (output, error, retz) = runShellCommand(fcomm)

        if(retz == 0):
            self.etime = 'INT_CPU_TIME'
            print self.etime
            killfiles(['tmpf.f', 'tmpf'])
            return
        

        writefile('tmpf.f',"""
      PROGRAM DSECND
      REAL               T1
      REAL               TARRAY( 2 )
      REAL               ETIME
      INTRINSIC          ETIME
      T1 = ETIME( TARRAY )
      END\n""")
        
        fcomm = self.mpif77+' '+self.fcflags+' -o tmpf tmpf.f'
        (output, error, retz) = runShellCommand(fcomm)

        if(retz == 0):
            self.etime = 'INT_ETIME'
            print self.etime
            killfiles(['tmpf.f', 'tmpf'])
            return


        killfiles(['tmpf.f', 'tmpf'])
        self.etime = 'NONE'



    def write_makeinc(self):
        """ Writes the make.inc file for LAPACK installation """

        print 'Writing make.inc...',
        writefile('make.inc',"""
SHELL = /bin/sh
PLAT =
FORTRAN  = """+self.mpif77+"""
OPTS     = """+self.fcflags+"""
DRVOPTS  = $(OPTS)
NOOPT    = """+self.noopt+"""
LOADER   = """+self.mpif77+"""
LOADOPTS =
TIMER    = """+self.etime+"""
ARCH     = ar
ARCHFLAGS= cr
RANLIB   = """+self.ranlib+"""
BLASLIB      = """+self.blaslib+"""
LAPACKLIB    = libreflapack.a
TMGLIB       = tmglib.a
EIGSRCLIB    = eigsrc.a
LINSRCLIB    = linsrc.a
        """)
        print 'done.'



#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------



from utils import writefile, runShellCommand, killfiles, downloader, getURLName
import sys
import os
from framework import Frame


class Scalapack(Frame):
    """ This class takes care of the BLACS installation. """
    def __init__(self):
        print '\n','='*40
        print 'ScaLAPACK installer is starting now. Buckle up!'
        print '='*40

        self.down_install()






    def write_slmakeinc(self):
        """ Writes the SLmake.inc file for ScaLAPACK installation """

        sdir = os.getcwd()
        print 'Writing SLmake.inc...',
        sys.stdout.flush()
        writefile('SLmake.inc',"""
SHELL         = /bin/sh

home          = """+sdir+"""

PLAT          = """+self.plat+"""

USEMPI        = -DUsingMpiBlacs

SMPLIB        = 
BLACSFINIT    = """+self.blacsF77lib+"""
BLACSCINIT    = """+self.blacsClib+"""
BLACSLIB      = """+self.blacslib+"""
TESTINGdir    = $(home)/TESTING


CBLACSLIB     = $(BLACSCINIT) $(BLACSLIB) $(BLACSCINIT)
FBLACSLIB     = $(BLACSFINIT) $(BLACSLIB) $(BLACSFINIT)

PBLASdir      = $(home)/PBLAS
SRCdir        = $(home)/SRC
TESTdir       = $(home)/TESTING
PBLASTSTdir   = $(TESTINGdir)
TOOLSdir      = $(home)/TOOLS
REDISTdir     = $(home)/REDIST
REDISTTSTdir  = $(TESTINGdir)

F77           = """+self.mpif77+"""
CC            = """+self.mpicc+"""
NOOPT         = """+self.noopt+"""
F77FLAGS      =  $(NOOPT) """+self.fcflags+"""
CCFLAGS       = """+self.ccflags+"""
SRCFLAG       =
F77LOADER     = $(F77) 
CCLOADER      = $(F77)
F77LOADFLAGS  = """+self.ldflags_f77+"""
CCLOADFLAGS   = """+self.ldflags_c+"""

CDEFS         = -DNO_IEEE $(USEMPI) """+self.mangling+"""

ARCH          = ar
ARCHFLAGS     = cr
RANLIB        = """+self.ranlib+"""

SCALAPACKLIB  = $(home)/libscalapack.a
BLASLIB       = """+self.blaslib+"""
LAPACKLIB     = """+self.lapacklib+"""

PBLIBS        = $(SCALAPACKLIB) $(FBLACSLIB) $(LAPACKLIB) $(BLASLIB) $(SMPLIB)
PRLIBS        = $(SCALAPACKLIB) $(CBLACSLIB) $(SMPLIB)
RLIBS         = $(SCALAPACKLIB) $(FBLACSLIB) $(CBLACSLIB) $(LAPACKLIB) $(BLASLIB) $(SMPLIB)
LIBS          = $(PBLIBS)
        """)
        print 'done.'






    def down_install(self):
        """ Downloads ind installs ScaLAPACK """

        savecwd = os.getcwd()

        # creating the build and lib dirs if don't exist
        if(not os.path.isdir(os.path.join(os.getcwd(),'build'))):
            os.mkdir(os.path.join(os.getcwd(),'build'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'lib'))):
            os.mkdir(os.path.join(os.getcwd(),'lib'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'log'))):
            os.mkdir(os.path.join(os.getcwd(),'log'))

        # chdir into the build directory            
        os.chdir(os.path.join(os.getcwd(),'build'))

        if(not os.path.isfile(os.path.join(os.getcwd(),getURLName(self.scalapackurl)))):
            downloader(self.scalapackurl, self.downcmd)

        comm = 'gunzip -f scalapack-1.8.0.tgz'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nScaLAPACK: cannot unzip scalapack-1.8.0.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        

        comm = 'tar xf scalapack-1.8.0.tar'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nScaLAPACK: cannot untar scalapack-1.8.0.tar'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        os.remove('scalapack-1.8.0.tar')
        
        os.chdir(os.path.join(os.getcwd(),'scalapack-1.8.0'))

        self.write_slmakeinc()

        print 'Compiling ScaLAPACK...',
        sys.stdout.flush()
        comm = self.make
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nScaLAPACK: error building ScaLAPACK'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            writefile(os.path.join(savecwd,'log/scalog'), output+error)
            sys.exit()

        fulllog = os.path.join(savecwd,'log/scalog')
        writefile(fulllog, output+error)
        print 'Installation of ScaLAPACK successful..'
        print '(log is in ',fulllog,')'
		
        if(self.testing == 1):
            print 'Compiling test routines...'
            sys.stdout.flush()
            comm = self.make+' exe'
            (output, error, retz) = runShellCommand(comm)
            if(retz != 0):
                print '\n\nScaLAPACK: error building ScaLAPACK test routines'
                print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
                writefile(os.path.join(savecwd,'log/scalog'), output+error)
                sys.exit()
            print 'done'

        
        os.rename('libscalapack.a',os.path.join(savecwd,'lib/libscalapack.a'))

        self.scalapacklib  = os.path.join(savecwd,'lib/libscalapack.a ')
        Frame.scalapacklib = os.path.join(savecwd,'lib/libscalapack.a ')

        os.chdir(savecwd)
        print "done. ScaLAPACK is installed. Use it in moderation :-)"



#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------

from utils import writefile, runShellCommand, killfiles, downloader, getURLName
import sys
import os
from framework import Frame


class Blas(Frame):
    """ This class takes care of the BLACS installation. """
    def __init__(self):
        print '\n','='*40
        print '  BLAS installation/verification'
        print '='*40

        if(self.blaslib != ''):
            self.check_blas()
        else:
            if (self.downblas):
                self.down_install_blas()
            else:
                print "Need BLAS"
                sys.exit()
            




    def check_blas(self):

        print 'Checking if provided BLAS works...',
        # This function simply generates a FORTRAN program
        # that contains few calls to BLAS routine and then
        # checks if compilation, linking and execution are succesful

        sys.stdout.flush()
        writefile('tmpf.f',"""
      program ftest
      double precision da, dx(1)
      dx(1)=1
      da = 2
      call dscal(1,da,dx,1)
      stop
      end\n""")

        fcomm = self.mpif77+' -o tmpf '+'tmpf.f '+self.blaslib+' '+self.ldflags_f77
        (output, error, retz) = runShellCommand(fcomm)
        
        if(retz != 0):
            print '\n\nBLAS: provided BLAS cannot be used! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        comm = './tmpf'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLAS: provided BLAS cannot be used! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        killfiles(['tmpf.f','tmpf'])
        print 'yes'

        return 0;


    def down_install_blas(self):

        print """
The reference BLAS library is being installed.
Don't expect high performance from this reference library!
If you want performance, you need to use an optimized BLAS library and,
to avoid unnecessary complications, if you need to compile this optimized BLAS
library, use the same compiler you're using here."""
        sys.stdout.flush()

        savecwd = os.getcwd()

        # creating the build,lib and log dirs if don't exist
        if(not os.path.isdir(os.path.join(os.getcwd(),'build'))):
            os.mkdir(os.path.join(os.getcwd(),'build'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'lib'))):
            os.mkdir(os.path.join(os.getcwd(),'lib'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'log'))):
            os.mkdir(os.path.join(os.getcwd(),'log'))

        # chdir into the build directory            
        os.chdir(os.path.join(os.getcwd(),'build'))

        # Check if blas.tgz is already present in the working dir
        # otherwise download it
        if(not os.path.isfile(os.path.join(os.getcwd(),getURLName(self.blasurl)))):
			print 'Downloading reference BLAS...',
			downloader(self.blasurl,self.downcmd)
			print 'done'
    
        # unzip and untar      
        print 'Unzip and untar reference BLAS...',
        comm = 'gunzip -f blas.tgz'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLAS: cannot unzip blas.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        comm = 'tar xf blas.tar'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLAS: cannot untar blas.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        os.remove('blas.tar')
        print 'done'
     
        # change to BLAS dir
        os.chdir(os.path.join(os.getcwd(),'BLAS'))

        # compile and generate library
        print 'Compile and generate reference BLAS...',
        sys.stdout.flush()
        comm = self.mpif77+' '+self.fcflags+' -c *.f'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLAS: cannot compile blas'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        log = output+error

        comm = 'ar cr librefblas.a *.o'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLAS: cannot create blas library'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        print 'done'

        # write the log on a file
        log = log+output+error
        fulllog = os.path.join(savecwd,'log/blaslog')
        writefile(fulllog, log)
        print 'Installation of reference BLAS successful.'
        print '(log is in ',fulllog,')'

        # move librefblas.a to the lib directory        
        os.rename('librefblas.a',os.path.join(savecwd,'lib/librefblas.a'))

        # set framework variables to point to the freshly installed BLAS library
        self.blaslib  = os.path.join(savecwd,'lib/librefblas.a ')
        Frame.blaslib = os.path.join(savecwd,'lib/librefblas.a ')
        os.chdir(savecwd)



#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------

import sys
import os

srcdir = os.getcwd()+'/script'
sys.path = sys.path+[srcdir]

from framework import Frame
from blas import Blas
from lapack import Lapack
from blacs import Blacs
from scalapack import Scalapack



if __name__ == '__main__':

    frame = Frame(sys.argv[1:])


    blas_inst     = Blas();
    lap_inst      = Lapack();
    blacs_inst    = Blacs();
    scal_inst     = Scalapack()

    Frame.resume(frame)



#!/usr/bin/python

# -----------------------------------------
# ScaLAPACK installer
# University of Tennessee Knoxville
# October 16, 2007
# ----------------------------------------

from utils import writefile, runShellCommand, killfiles, downloader, getURLName
import sys
import os
from framework import Frame


class Blacs(Frame):
    """ This class takes care of the BLACS installation. """
    def __init__(self):
        print '\n','='*40
        print '  BLACS installation/verification'
        print '='*40

        if (self.downblacs):
            self.down_install_blacs()
        elif(self.blacslib != '' and self.blacsClib!='' and self.blacsF77lib !=''):
            self.check_blacs()
        else:
            print 'need blacs'
            sys.exit()



    def check_blacs(self):
        """ Checks if BLACS libraries are good """

        # This function simply generates a FORTRAN program
        # that contains few calls to BLACS routine and then
        # checks if compilation, linking and execution are succesful
        print 'Checking if provided BLACS works...',
        sys.stdout.flush()
        writefile('tmpf.f',"""
      program ftest
      integer itmp, iam, nnodes
      call blacs_pinfo(iam, nnodes)
      if(nnodes.gt.0) then
         call blacs_get( 0, 0, itmp )
         call blacs_gridinit(itmp, 'c', 1, nnodes)
         call blacs_gridexit(itmp)
      end if
      stop
      end\n""")

        fcomm = self.mpif77+' '+self.ldflags_f77+' -o tmpf '+'tmpf.f '+self.blacsF77lib+' '+self.blacslib+' '+self.blacsClib
        (output, error, retz) = runShellCommand(fcomm)
        
        if(retz != 0):
            print '\n\nBLACS: provided BLACS cannot be used! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        comm = './tmpf'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLACS: provided BLACS cannot be used! aborting...'
            print 'error is:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        killfiles(['tmpf.f','tmpf'])
        print 'yes'

        return 0;



    def down_install_blacs(self):
        """ Downloads and installs BLACS libraries """

        savecwd = os.getcwd()

        # creating the build and lib dirs if don't exist
        if(not os.path.isdir(os.path.join(os.getcwd(),'build'))):
            os.mkdir(os.path.join(os.getcwd(),'build'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'lib'))):
            os.mkdir(os.path.join(os.getcwd(),'lib'))

        if(not os.path.isdir(os.path.join(os.getcwd(),'log'))):
            os.mkdir(os.path.join(os.getcwd(),'log'))

        # chdir into the build directory            
        os.chdir(os.path.join(os.getcwd(),'build'))
        # checks if mpiblacs.tgz is already present in the current directory
        # and if not it downloads it
        if(not os.path.isfile(os.path.join(os.getcwd(),getURLName(self.blacsurl)))):
            print 'Downloading BLACS...',
            downloader(self.blacsurl, self.downcmd)
            print 'done'

        # unzip and untar            
        print 'Unzip and untar BLACS...',
        comm = 'gunzip -f mpiblacs.tgz'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLACS: cannot unzip mpiblacs.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        

        comm = 'tar xf mpiblacs.tar'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLACS: cannot untar mpiblacs.tgz'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        os.remove('mpiblacs.tar')
        print 'done'
        
        # goes in the BLACS dir
        os.chdir(os.path.join(os.getcwd(),'BLACS'))

        # set TRANSCOMM
        self.set_transcomm()
        # write Bmake.inc file
        self.write_bmake()

        # compile
        print 'Compiling BLACS...',
        sys.stdout.flush()
        comm = self.make+' mpi'
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLACS: error building BLACS'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()
        
        # write a log file containing the BLACS compilation output
        fulllog = os.path.join(savecwd,'log/blacslog')
        writefile(fulllog, output+error)
        print 'done.'
        print ' (log is in ',fulllog,')'

        os.rename('LIB/blacs.a',os.path.join(savecwd,'lib/blacs.a'))
        os.rename('LIB/blacsC.a',os.path.join(savecwd,'lib/blacsC.a'))
        os.rename('LIB/blacsF77.a',os.path.join(savecwd,'lib/blacsF77.a'))

        # sets framework variables to point to the freshly installed BLACS libraries
        self.blacslib     = os.path.join(savecwd,'lib/blacs.a ')
        self.blacsClib    = os.path.join(savecwd,'lib/blacsC.a ')
        self.blacsF77lib  = os.path.join(savecwd,'lib/blacsF77.a ')
        Frame.blacslib    = os.path.join(savecwd,'lib/blacs.a ')
        Frame.blacsClib   = os.path.join(savecwd,'lib/blacsC.a ')
        Frame.blacsF77lib = os.path.join(savecwd,'lib/blacsF77.a ')
        os.chdir(savecwd)






    def set_transcomm(self):
        """ Sets the TRANSCOMM variable in Bmake.inc """
        # This one compiles few programs equivalent to those in BLACS/INSTALL
        # in order to set the TRANSCOMM value

        print 'Setting TRANSCOMM...', 
        sys.stdout.flush()

        writefile('tmpf.f',"""
      program tctst
      include 'mpif.h'
      integer i, ierr
      external Ccommcheck
      integer  Ccommcheck

      call mpi_init(ierr)

      i = Ccommcheck(MPI_COMM_WORLD, MPI_COMM_SELF)
      if (i .ne. 0) then
         print*,'-DCSameF77'
      end if

      call mpi_finalize(ierr)

      stop
      end\n""")

        writefile('tmpc.c',"""
#include <mpi.h>

int Ccommcheck(int F77World, int f77comm){
   int Np, Iam, i, OK=1;

   if (sizeof(int) != sizeof(MPI_Comm)) OK=0;
   else if ((MPI_Comm) F77World != MPI_COMM_WORLD) OK=0;
   else {
      MPI_Comm_rank(MPI_COMM_WORLD, &Iam);
      i = MPI_Comm_size((MPI_Comm) f77comm, &Np);
      if (i != MPI_SUCCESS) OK = 0;
      else if (Np != 1) OK = 0;
   }
   return(OK);
}
int CCOMMCHECK(int *F77World, int *f77comm){ return(Ccommcheck(*F77World, *f77comm));}
int ccommcheck_(int *F77World, int *f77comm){return(Ccommcheck(*F77World, *f77comm));}
int ccommcheck(int *F77World, int *f77comm){return(Ccommcheck(*F77World, *f77comm));}\n""")
        
        ccomm = self.mpicc+' '+self.ccflags+' -c tmpc.c -o tmpc.o'
        fcomm = self.mpif77+' '+self.fcflags+' tmpf.f tmpc.o -o xtc_CsameF77'

        (output, error, retz) = runShellCommand(ccomm)
        if(retz != 0):
            print '\n\nBLACS: Error in transcomm setting! cannot compile'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        (output, error, retz) = runShellCommand(fcomm)
        if(retz != 0):
            print '\n\n1BLACS: Error in transcomm setting! cannot compile'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        comm = os.path.join(os.getcwd(),'xtc_CsameF77')
        (output, error, retz) = runShellCommand(comm)
        if(retz != 0):
            print '\n\nBLACS: Error in transcomm setting! cannot run xtc_CsameF77'
            print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
            sys.exit()

        killfiles(['tmpf.f', 'tmpc.c', 'tmpc.o', 'xtc_CsameF77'])
        
        self.transcomm = output


        if(self.transcomm == ''):
            writefile('tmpc.c',"""
            #include <stdio.h>
            #include <mpi.h>
            main(){
               MPI_Comm ccomm;
               int fcomm;
               extern void *MPIR_ToPointer();
               extern int   MPIR_FromPointer();
               extern void *MPIR_RmPointer();
            
               if (sizeof(int) < sizeof(int*))   {
                  fcomm = MPIR_FromPointer(MPI_COMM_WORLD);
                  ccomm = (MPI_Comm) MPIR_ToPointer(fcomm);
                  if (ccomm == MPI_COMM_WORLD)
                     fprintf(stdout,\" -DUseMpich -DPOINTER_64_BITS=1\");
               }
               return 0;
            }\n""")
        
            ccomm = self.mpicc+' '+self.ccflags+' tmpc.c -o UseMpich'
            (output, error, retz) = runShellCommand(ccomm)
            if(retz != 0):
                print self.transcomm
                return 1;
            
            comm = os.path.join(os.getcwd(),'UseMpich')
            (output, error, retz) = runShellCommand(comm)
            if(retz != 0):
                print '\n\nBLACS: Error in transcomm setting! cannot run UseMpich'
                print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
                sys.exit()

            killfiles(['tmpc.c','UseMpich'])
            self.transcomm = output

# Testing MPI2 implementation
        if(self.transcomm == ''):
	            writefile('tmpc.c',"""
	            #include <stdio.h>
	            #include <mpi.h>
				int main(int argc, char** argv)
				{
				  int version, subversion;
				  MPI_Init(&argc, &argv);
				  MPI_Get_version(&version, &subversion);
				  if (version==2) fprintf(stdout,\" -DUseMpi2\");
				  MPI_Finalize();
				  return 0;
	            }\n""")

	            ccomm = self.mpicc+' '+self.ccflags+' tmpc.c -o UseMPI2'
	            (output, error, retz) = runShellCommand(ccomm)
	            if(retz != 0):
	                print self.transcomm
	                print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
	                sys.exit()

	            comm = os.path.join(os.getcwd(),'UseMPI2')
	            (output, error, retz) = runShellCommand(comm)
	            if(retz != 0):
	                print '\n\nBLACS: Error in transcomm setting! cannot run UseMPI2'
	                print 'stderr:\n','*'*40,'\n',error,'\n','*'*40
	                sys.exit()

	            killfiles(['tmpc.c','UseMPI2'])
	            self.transcomm = output
	            
        print self.transcomm

            
        return 1;
        

    def write_bmake(self):
      """ Writes the Bmake.inc file for BLACS installation """
      print 'Writing Bmake.inc...',
      sys.stdout.flush()
      writefile('Bmake.inc',"""
SHELL = /bin/sh
BTOPdir = """+os.getcwd()+"""
COMMLIB = MPI
PLAT = 
BLACSdir    = $(BTOPdir)/LIB
BLACSDBGLVL = 0
BLACSFINIT  = $(BLACSdir)/blacsF77.a
BLACSCINIT  = $(BLACSdir)/blacsC.a
BLACSLIB    = $(BLACSdir)/blacs.a

MPIINCdir = """+self.mpiincdir+"""
MPILIB = 

BTLIBS = $(BLACSFINIT) $(BLACSLIB) $(BLACSFINIT) $(MPILIB)
INSTdir = $(BTOPdir)/INSTALL/EXE
TESTdir = $(BTOPdir)/TESTING/EXE
FTESTexe = $(TESTdir)/xFbtest_$(COMMLIB)-$(PLAT)-$(BLACSDBGLVL)
CTESTexe = $(TESTdir)/xCbtest_$(COMMLIB)-$(PLAT)-$(BLACSDBGLVL)
SYSINC = -I$(MPIINCdir)
INTFACE = """+self.mangling+"""
SENDIS = 
BUF = 
TRANSCOMM = """+self.transcomm+"""
WHATMPI = 
SYSERRORS = 
DEBUGLVL = -DBlacsDebugLvl=$(BLACSDBGLVL)
DEFS1 = -DSYSINC $(SYSINC) $(INTFACE) $(DEFBSTOP) $(DEFCOMBTOP) $(DEBUGLVL)
BLACSDEFS = $(DEFS1) $(SENDIS) $(BUFF) $(TRANSCOMM) $(WHATMPI) $(SYSERRORS)

F77            = """+self.mpif77+"""
F77NO_OPTFLAGS = """+self.noopt+"""
F77FLAGS       = $(F77NO_OPTFLAGS) """+self.fcflags+"""
F77LOADER      = $(F77)
F77LOADFLAGS   = """+self.ldflags_f77+"""
CC             = """+self.mpicc+"""
CCFLAGS        = """+self.ccflags+"""
CCLOADER       = $(F77)
CCLOADFLAGS    = """+self.ldflags_c+"""
ARCH      = ar
ARCHFLAGS = r
RANLIB    = """+self.ranlib+"""
      """)
      print 'done.'
      return 1;

